// main.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "news.h"

using namespace std;

typedef vector<CNews*> ItemsVector;

///////////////////////////////////////////////////////////////////////////////
//
// Exported functions
//
extern "C" {
    __declspec( dllexport ) int initModuleEx(HWND hwndParent, HINSTANCE hDllInstance, LPCSTR szPath);
    __declspec( dllexport ) void quitModule(HINSTANCE hDllInstance);
}

///////////////////////////////////////////////////////////////////////////////
//
// News description items
//
#define CONFIG_TOKENS_NUM	2
ItemsVector handles;
CNews* GetItemByName(LPCTSTR name);

///////////////////////////////////////////////////////////////////////////////
//
// Class and application names,
// revision id support and common messages handling
//
const char szAppName[]		= "Wazup";
const char szMsgHandler[]	= "LSWazupManager";
const char rcsRevision[]	= "1.0";

int lsMessages[] =
{
	LM_GETREVID,
	0
};
HWND messageHandler;
void CreateMessageHandler(HINSTANCE hInst);
LRESULT WINAPI MessageHandlerProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

UINT_PTR LastEventIDValue = 0;

///////////////////////////////////////////////////////////////////////////////
//
// Exported !bangs
//
#define EXPORTED_BANGS_NUM	8

void bangCheck(HWND caller, LPCTSTR szArgs);
void bangEnable(HWND caller, LPCTSTR szArgs);
void bangDisable(HWND caller, LPCTSTR szArgs);
void bangToggle(HWND caller, LPCTSTR szArgs);
void bangSetURL(HWND caller, LPCTSTR szArgs);
void bangSetInputString(HWND caller, LPCTSTR szArgs);
void bangSetOutputString(HWND caller, LPCTSTR szArgs);
void bangSetUpdateInterval(HWND caller, LPCTSTR szArgs);

struct BangItem
{
	char name[25];
	BangCommand *command;
} bangs[] = 
{
	{"!WazupCheck",				bangCheck},
	{"!WazupEnable",			bangEnable},
	{"!WazupDisable",			bangDisable},
	{"!WazupToggle",			bangToggle},
	{"!WazupSetURL",			bangSetURL},
	{"!WazupSetInputString",	bangSetInputString},
	{"!WazupSetOutputString",	bangSetOutputString},
	{"!WazupSetUpdateInterval",	bangSetUpdateInterval}
};


///////////////////////////////////////////////////////////////////////////////
//
// DLL entry point
//
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{

    return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
//
// Error Box
//
void ErrorBox(const char *szMessage)
{
	MessageBox(NULL, szMessage, "LiteStep :: Wazup", MB_OK | MB_ICONERROR | MB_TOPMOST | MB_SETFOREGROUND);
	LSLogPrintf(LOG_ERROR, szAppName, szMessage);
}


///////////////////////////////////////////////////////////////////////////////
//
// Module entry point
//
int initModuleEx(HWND hwndParent, HINSTANCE hDllInstance, LPCSTR szPath)
{
	LSLogPrintf(LOG_DEBUG, szAppName, "Initializing");
	CreateMessageHandler(hDllInstance);

	FILE *f;
	if(f = LCOpen(NULL))
	{
		char line[MAX_LINE_LENGTH];	// current line

		// let's prepare string array for tokens
		char token[CONFIG_TOKENS_NUM][MAX_LINE_LENGTH];
		char* tokens[CONFIG_TOKENS_NUM];
		char szExtra[MAX_LINE_LENGTH];

		for (int i=0; i<CONFIG_TOKENS_NUM; i++)
		{
			tokens[i] = token[i];
		}

		// enumerate all news description items
		while(LCReadNextConfig(f, "*Wazup", line, MAX_LINE_LENGTH))
		{
			LSLogPrintf(LOG_DEBUG, szAppName, "Luke Skywalker!");
			LCTokenize(line, tokens, CONFIG_TOKENS_NUM, szExtra);
			if (token[1][0] == 0)
				ErrorBox("Invalid config: *Wazup string without name found");
			else
			{
				CNews *info = new CNews(token[1]);
				if (!info->ReadSettings())
					delete info;
				else
				{
					handles.push_back(info);
					if (info->m_sLocalFile != "")
					{
						info->ReadFromFile(info->m_sLocalFile.c_str(), false);
					}
					if (info->m_bActive)
					{
						info->m_bActive = false;
						info->Enable();
						info->Update();
					}
				}
			}
		}
		LCClose(f);

		// Export !bangs
		for (i=0; i<EXPORTED_BANGS_NUM; i++)
		{
			AddBangCommand(bangs[i].name, bangs[i].command);
		}
	} else ErrorBox("Error: Cannot open step.rc for reading");
	LSLogPrintf(LOG_DEBUG, szAppName, "Module has been started");
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
// Module destructor
//
void quitModule(HINSTANCE hDllInstance)
{
	LSLogPrintf(LOG_DEBUG, szAppName, "Destructor called");
	for (int i=0; i<EXPORTED_BANGS_NUM; i++)
	{
		RemoveBangCommand(bangs[i].name);
	}
	for (ItemsVector::iterator ri = handles.begin(); ri != handles.end(); ri++)
	{
		delete *ri;
	}
	handles.clear();
	SendMessage(GetLitestepWnd(),LM_UNREGISTERMESSAGE,(WPARAM) messageHandler,(LPARAM) lsMessages);
	if (messageHandler)
	{
		DestroyWindow(messageHandler);
	}
	UnregisterClass(szMsgHandler, hDllInstance);
	LSLogPrintf(LOG_DEBUG, szAppName, "Module has been unloaded");
	return;
}

////////////////////////////////////////////////////////////////////////////
//
// Create message handler window
//

void CreateMessageHandler(HINSTANCE hInst)
{
	WNDCLASSEX wc;
	ZeroMemory(&wc, sizeof(WNDCLASSEX));
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.lpfnWndProc = MessageHandlerProc;
	wc.hInstance = hInst;
	wc.lpszClassName = szMsgHandler;
	wc.hIconSm = 0;

	if (!RegisterClassEx(&wc))
	{
		ErrorBox("Error: Cannot register handler window class...");
		return;
	}

	messageHandler = CreateWindowEx(WS_EX_TOOLWINDOW,
		szMsgHandler,
		0,
		WS_POPUP,
		0, 0, 0, 0, 
		0,
		0,
		hInst,
		0);

	if (!messageHandler)
	{
		ErrorBox("Error: Cannot create main window...");
		return;
	}	
	SendMessage(GetLitestepWnd(), LM_REGISTERMESSAGE, (WPARAM) messageHandler, (LPARAM) lsMessages);

}


////////////////////////////////////////////////////////////////////////////
//
// Message handler would process the following messages
//	LM_GETREVID		-	Revision ID
//	WM_TIMER		-	news update timer
//
LRESULT WINAPI MessageHandlerProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
	case LM_GETREVID:
		LPSTR buf;
		buf = (LPSTR)lParam;
		switch (wParam)
		{
		case 0:
		case 1:
			StringCchPrintf(buf, lParam, "Wazup: %s (Seg@)", rcsRevision);
			break;
		default:
			StringCchCopy(buf, lParam, "");
		}
		return strlen(buf);
	case WM_TIMER:
		ItemsVector::iterator ri = handles.begin(), rend = handles.end();
		for (; ri != rend; ri++)
		{
			if ((*ri)->m_nIDEvent == wParam) (*ri)->Update();
		}
		break;
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}


///////////////////////////////////////////////////////////////////
//
// Find the news import group with the specified name
//
CNews* GetItemByName(LPCTSTR name)
{
	char tmp[1024];
	StringCchCopy(tmp, 1024, (name[0] == '\"') ? name + 1 : name);
	if (tmp[strlen(tmp)-1] == '\"')
	{
		tmp[strlen(tmp)-1] = 0;
	}
	for (ItemsVector::iterator qi = handles.begin(); qi != handles.end(); qi++)
	{
		if (stricmp((*qi)->m_sName.c_str(), tmp) == 0)
		{
			return (*qi);
		}
	}
	return NULL;
}

////////////////////////////////////////////////////////////////////////////
//
// Diagnostic message for !bangs
//
void BangFailed(const char *szBangName, const char *szNewsItem)
{
	char errmsg[1024];
	StringCchPrintf(errmsg, 1024,
		"News description item with name \"%s\" not found\n%s failed", szBangName, szNewsItem);
	ErrorBox(errmsg);
}

////////////////////////////////////////////////////////////////////////////
//
// Bangs
//	Check				- check the page for updates just now
//	Disable				- disable automatic updates
//	Enable				- enable automatic updates
//	Toggle				- toggle updating
//	SetURL				- set web-page URL
//	SetInputString		- set the title formatting string
//	SetOutputString		- set the items formatting string
//	SetUpdateInterval	- set an interval between automatic updates
//

void bangCheck(HWND caller, LPCTSTR szArgs)
{
	CNews *info = GetItemByName(szArgs);
	if (info)
		info->Update();
	else
		BangFailed("!WazupUpdate", szArgs);
}

void bangEnable(HWND caller, LPCTSTR szArgs)
{
	CNews *info = GetItemByName(szArgs);
	if (info)
		info->Enable();
	else
		BangFailed("!WazupEnable", szArgs);
}

void bangDisable(HWND caller, LPCTSTR szArgs)
{
	CNews *info = GetItemByName(szArgs);
	if (info)
		info->Disable();
	else
		BangFailed("!WazupDisable", szArgs);
}

void bangToggle(HWND caller, LPCTSTR szArgs)
{
	CNews *info = GetItemByName(szArgs);
	if (info)
		info->Toggle();
	else
		BangFailed("!WazupToggle", szArgs);
}

void bangSetURL(HWND caller, LPCTSTR szArgs)
{
	char token1[MAX_LINE_LENGTH], token2[MAX_LINE_LENGTH], szExtra[MAX_LINE_LENGTH];
	char *tokens[2] = {token1, token2};
	LCTokenize(szArgs, tokens, 2, szExtra);
	CNews *info = GetItemByName(token1);
	if (info)
		info->m_sURL = token2;
	else
		BangFailed("!WazupSetURL", token1);
}

void bangSetInputString(HWND caller, LPCTSTR szArgs)
{
	char token1[MAX_LINE_LENGTH], token2[MAX_LINE_LENGTH], szExtra[MAX_LINE_LENGTH];
	char *tokens[2] = {token1, token2};
	LCTokenize(szArgs, tokens, 2, szExtra);
	CNews *info = GetItemByName(token1);
	if (info)
		info->m_sInputString = token2;
	else
		BangFailed("!WazupSetInputString", token1);
}

void bangSetOutputString(HWND caller, LPCTSTR szArgs)
{
	char token1[MAX_LINE_LENGTH], token2[MAX_LINE_LENGTH], szExtra[MAX_LINE_LENGTH];
	char *tokens[2] = {token1, token2};
	LCTokenize(szArgs, tokens, 2, szExtra);
	CNews *info = GetItemByName(token1);
	if (info)
		info->m_sOutputString = token2;
	else
		BangFailed("!WazupSetOutputString", token1);
}

void bangSetUpdateInterval(HWND caller, LPCTSTR szArgs)
{
	char token1[MAX_LINE_LENGTH], token2[MAX_LINE_LENGTH], szExtra[MAX_LINE_LENGTH];
	char *tokens[2] = {token1, token2};
	LCTokenize(szArgs, tokens, 2, szExtra);
	CNews *info = GetItemByName(token1);
	if (info == NULL)
		BangFailed("!WazupSetUpdateInterval", token1);
	else
	{
		char *err;
		int interval = strtod(token2, &err);
		if (err == 0 && interval>0)
		{
			info->m_UpdateInterval = interval;
			if (info->m_bActive)
			{
				KillTimer(messageHandler, info->m_uTimerID);
				info->m_uTimerID = SetTimer(messageHandler, info->m_nIDEvent, 1000*interval, NULL);
			}
		}
	}
}
